"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.serviceCtlFsmMixin = exports.ServiceCtlFsm = void 0;
/**
 * Licenst: Apache-2.0
 * https://github.com/huan/state-switch
 */
const events_1 = require("events");
const xstate_1 = require("xstate");
const version_js_1 = require("../version.js");
// import { nopLogger }            from '../nop-logger.js'
const machine_config_js_1 = require("./machine-config.js");
const machine_options_js_1 = require("./machine-options.js");
const wait_for_selector_js_1 = require("./wait-for-selector.js");
const guard_machine_event_js_1 = require("./guard-machine-event.js");
const nop_logger_js_1 = require("../nop-logger.js");
const serviceCtlFsmMixin = (serviceCtlName = 'ServiceCtlFsm', options) => (superClass) => {
    class ServiceCtlFsmMixin extends superClass {
        constructor(...args) {
            super(...args);
            this._serviceCtlLog = options?.log || (0, nop_logger_js_1.nopLogger)();
            this._serviceCtlLog.verbose(serviceCtlName, 'constructor()');
            const machineOptions = (0, machine_options_js_1.buildMachineOptions)({
                // reset: () => has been internally implemented by calling stop() and start()
                start: () => this.onStart(),
                stop: () => this.onStop(),
            });
            const machine = (0, xstate_1.createMachine)(machine_config_js_1.config, machineOptions);
            this._serviceCtlFsmInterpreter = (0, xstate_1.interpret)(machine);
            this._serviceCtlFsmInterpreter.start();
        }
        start() {
            this._serviceCtlLog.verbose(serviceCtlName, 'start()');
            (0, guard_machine_event_js_1.guardMachineEvent)(this._serviceCtlFsmInterpreter, 'START');
            const started = (0, wait_for_selector_js_1.waitForMachineState)(this._serviceCtlFsmInterpreter, 'active');
            const canceled = (0, wait_for_selector_js_1.waitForMachineState)(this._serviceCtlFsmInterpreter, 'canceled');
            this._serviceCtlFsmInterpreter.send('START');
            return Promise.race([
                started,
                canceled,
            ]);
        }
        stop() {
            this._serviceCtlLog.verbose(serviceCtlName, 'stop()');
            (0, guard_machine_event_js_1.guardMachineEvent)(this._serviceCtlFsmInterpreter, 'STOP');
            const stopped = (0, wait_for_selector_js_1.waitForMachineState)(this._serviceCtlFsmInterpreter, 'inactive');
            const canceled = (0, wait_for_selector_js_1.waitForMachineState)(this._serviceCtlFsmInterpreter, 'canceled');
            this._serviceCtlFsmInterpreter.send('STOP');
            return Promise.race([
                stopped,
                canceled,
            ]);
        }
        reset() {
            this._serviceCtlLog.verbose(serviceCtlName, 'reset()');
            (0, guard_machine_event_js_1.guardMachineEvent)(this._serviceCtlFsmInterpreter, 'RESET');
            const started = (0, wait_for_selector_js_1.waitForMachineState)(this._serviceCtlFsmInterpreter, 'active');
            const canceled = (0, wait_for_selector_js_1.waitForMachineState)(this._serviceCtlFsmInterpreter, 'canceled');
            this._serviceCtlFsmInterpreter.send('RESET');
            // TODO: emit('error' e) if there's any rejections inside `reset()`
            return Promise.race([
                started,
                canceled,
            ]);
        }
    }
    ServiceCtlFsmMixin.VERSION = version_js_1.VERSION;
    return ServiceCtlFsmMixin;
};
exports.serviceCtlFsmMixin = serviceCtlFsmMixin;
class ServiceCtlFsm extends serviceCtlFsmMixin()(events_1.EventEmitter) {
}
exports.ServiceCtlFsm = ServiceCtlFsm;
//# sourceMappingURL=service-ctl-fsm.js.map