#!/usr/bin/env -S node --no-warnings --loader ts-node/esm
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tstest_1 = require("tstest");
const service_ctl_fsm_js_1 = require("./service-ctl-fsm.js");
(0, tstest_1.test)('ServiceCtlFsm smoke testing', async (t) => {
    const sandbox = tstest_1.sinon.createSandbox();
    const onStartSpy = sandbox.spy();
    const onStopSpy = sandbox.spy();
    class ServiceCtlFsmImpl extends service_ctl_fsm_js_1.ServiceCtlFsm {
        async onStart() {
            onStartSpy();
        }
        async onStop() {
            onStopSpy();
        }
    }
    const ctl = new ServiceCtlFsmImpl();
    await ctl.start();
    t.ok(onStartSpy.calledOnce, 'should call onStart()');
    t.ok(onStopSpy.notCalled, 'should not call onStop()');
    await ctl.stop();
    t.ok(onStopSpy.calledOnce, 'should call onStop()');
    t.throws(() => ctl.reset(), 'should reject when calling reset() with an inactive service');
    await ctl.start();
    sandbox.resetHistory();
    await t.resolves(() => ctl.reset(), 'should be able to reset with an active service');
    t.ok(onStartSpy.calledOnce, 'should call onStart() via reset()');
    t.ok(onStopSpy.calledOnce, 'should call onStop() via reset()');
});
//# sourceMappingURL=service-ctl-fsm.spec.js.map