/**
 * Licenst: Apache-2.0
 * https://github.com/huan/state-switch
 */
import { EventEmitter } from 'events';
import { createMachine, interpret, } from 'xstate';
import { VERSION } from '../version.js';
// import { nopLogger }            from '../nop-logger.js'
import { config, } from './machine-config.js';
import { buildMachineOptions } from './machine-options.js';
import { waitForMachineState } from './wait-for-selector.js';
import { guardMachineEvent } from './guard-machine-event.js';
import { nopLogger } from '../nop-logger.js';
const serviceCtlFsmMixin = (serviceCtlName = 'ServiceCtlFsm', options) => (superClass) => {
    class ServiceCtlFsmMixin extends superClass {
        constructor(...args) {
            super(...args);
            this._serviceCtlLog = options?.log || nopLogger();
            this._serviceCtlLog.verbose(serviceCtlName, 'constructor()');
            const machineOptions = buildMachineOptions({
                // reset: () => has been internally implemented by calling stop() and start()
                start: () => this.onStart(),
                stop: () => this.onStop(),
            });
            const machine = createMachine(config, machineOptions);
            this._serviceCtlFsmInterpreter = interpret(machine);
            this._serviceCtlFsmInterpreter.start();
        }
        start() {
            this._serviceCtlLog.verbose(serviceCtlName, 'start()');
            guardMachineEvent(this._serviceCtlFsmInterpreter, 'START');
            const started = waitForMachineState(this._serviceCtlFsmInterpreter, 'active');
            const canceled = waitForMachineState(this._serviceCtlFsmInterpreter, 'canceled');
            this._serviceCtlFsmInterpreter.send('START');
            return Promise.race([
                started,
                canceled,
            ]);
        }
        stop() {
            this._serviceCtlLog.verbose(serviceCtlName, 'stop()');
            guardMachineEvent(this._serviceCtlFsmInterpreter, 'STOP');
            const stopped = waitForMachineState(this._serviceCtlFsmInterpreter, 'inactive');
            const canceled = waitForMachineState(this._serviceCtlFsmInterpreter, 'canceled');
            this._serviceCtlFsmInterpreter.send('STOP');
            return Promise.race([
                stopped,
                canceled,
            ]);
        }
        reset() {
            this._serviceCtlLog.verbose(serviceCtlName, 'reset()');
            guardMachineEvent(this._serviceCtlFsmInterpreter, 'RESET');
            const started = waitForMachineState(this._serviceCtlFsmInterpreter, 'active');
            const canceled = waitForMachineState(this._serviceCtlFsmInterpreter, 'canceled');
            this._serviceCtlFsmInterpreter.send('RESET');
            // TODO: emit('error' e) if there's any rejections inside `reset()`
            return Promise.race([
                started,
                canceled,
            ]);
        }
    }
    ServiceCtlFsmMixin.VERSION = VERSION;
    return ServiceCtlFsmMixin;
};
class ServiceCtlFsm extends serviceCtlFsmMixin()(EventEmitter) {
}
export { ServiceCtlFsm, serviceCtlFsmMixin, };
//# sourceMappingURL=service-ctl-fsm.js.map